import json


class ControlPointProtocol:

    UDP_SEND_MSG = 'ZIMI'
    UDP_SEND_PORT = 5001
    UDP_RECV_PORT = 5002

    @classmethod
    def get(cls, target: str) -> str:
        '''Returns the get messages assocated with a target'''
        request = {"request": {
            "path": "api/v1/controlpoint/", "method": "GET"}}
        if target in ('actions', 'properties', 'states'):
            request["request"]["path"] += target
            return json.dumps(request)
        else:
            raise KeyError(
                'Only "actions", "properties" and "states" are supported')

    @classmethod
    def get_response_key(cls, target: str) -> str:
        '''Returns the key used to mark responses to target gets'''
        if target in ('actions', 'properties', 'states'):
            return "controlpoint_" + target
        else:
            raise KeyError(
                'Only "actions", "properties" and "states" are supported')

    @classmethod
    def set(cls, identifier: str, action: str, params: object) -> str:
        '''Returns the set messages assocated with an action'''
        request = {"request": {
            "path": "api/v1/controlpoint/actions", "method": "POST"}}
        if action in ("TurnOn", "TurnOff", "OpenDoor", "CloseDoor"):
            body = {"actions": [{"id": identifier, "action": action}]}
            request["request"]["body"] = body
        elif action in ("Fade", "OpenToPercentage", "SetBrightness", "SetFanSpeed") and params:
            body = {"actions": [
                {"id": identifier,
                 "action": action,
                 "actionParams": params}]}
            request["request"]["body"] = body
        else:
            raise KeyError('Combination of input parameters are not supported')
        return json.dumps(request)

    @classmethod
    def set_response_key(cls) -> str:
        '''Returns the key used to mark responses for actions'''
        return 'controlpoint_states_events'

    @classmethod
    def subscribe(cls) -> str:
        '''Returns the message assocated with state subscribe'''
        request = {"request": {
            "path": "api/v1/subscribe/controlpoint/states", "method": "POST"}}
        return json.dumps(request)

    @classmethod
    def subscribe_response_key(cls) -> str:
        '''Returns the key used to mark responses for subscribe'''
        return 'controlpoint_states_events'

    @classmethod
    def unsubscribe(cls) -> str:
        '''Returns the message assocated with state subscribe'''
        request = {"request": {
            "path": "api/v1/unsubscribe/controlpoint/states", "method": "POST"}}
        return json.dumps(request)
